@@ -14,6 +14,7 @@ from .gateways import make_call, send_sms
 
 
 phone_number_validator = RegexValidator(
+    code='invalid-phone-number',
     regex='^(\+|00)',
     message=_('Please enter a valid phone number, including your country code '
               'starting with + or 00.'),
@@ -28,9 +29,9 @@ PHONE_METHODS = (
 def get_available_phone_methods():
     methods = []
     if getattr(settings, 'TWO_FACTOR_CALL_GATEWAY', None):
-        methods.append(('call', _('Phone Call')))
+        methods.append(('call', _('Phone call')))
     if getattr(settings, 'TWO_FACTOR_SMS_GATEWAY', None):
-        methods.append(('sms', _('Text Message')))
+        methods.append(('sms', _('Text message')))
     return methods
 
 
