@@ -1,24 +1,30 @@
 from binascii import unhexlify
+from base64 import b32encode
+import qrcode
+import qrcode.image.svg
 
 from django.conf import settings
 from django.contrib.auth import login as login, REDIRECT_FIELD_NAME
 from django.contrib.auth.decorators import login_required
 from django.contrib.auth.forms import AuthenticationForm
 from django.contrib.sites.models import get_current_site
+from django.core.urlresolvers import reverse
 from django.forms import Form
+from django.http import HttpResponse, Http404
 from django.shortcuts import redirect
 from django.views.decorators.cache import never_cache
 from django.views.generic import FormView, DeleteView, TemplateView
+from django.views.generic.base import View
 from django_otp.decorators import otp_required
 from django_otp.plugins.otp_static.models import StaticToken
 from django_otp.util import random_hex
 
-from ..compat import is_safe_url
+from ..compat import is_safe_url, import_by_path
 from ..forms import (MethodForm, TOTPDeviceForm, PhoneNumberMethodForm,
                      DeviceValidationForm, AuthenticationTokenForm,
                      PhoneNumberForm, BackupTokenForm)
 from ..models import PhoneDevice
-from ..utils import (get_qr_url, default_device,
+from ..utils import (get_otpauth_url, default_device,
                      backup_phones)
 from .utils import (IdempotentSessionWizardView, class_view_decorator)
 
@@ -162,7 +168,10 @@ class SetupView(IdempotentSessionWizardView):
     is asked to provide a generated token. For call and sms methods, the user
     provides the phone number which is then validated in the final step.
     """
+    redirect_url = 'two_factor:setup_complete'
+    qrcode_url = 'two_factor:qr'
     template_name = 'two_factor/core/setup.html'
+    session_key_name = 'django_two_factor-qr_secret_key'
     initial_dict = {}
     form_list = (
         ('welcome', Form),
@@ -188,7 +197,7 @@ class SetupView(IdempotentSessionWizardView):
         Start the setup wizard. Redirect if already enabled.
         """
         if default_device(self.request.user):
-            return redirect('two_factor:setup_complete')
+            return redirect(self.redirect_url)
         return super(SetupView, self).get(request, *args, **kwargs)
 
     def render_next_step(self, form, **kwargs):
@@ -214,7 +223,7 @@ class SetupView(IdempotentSessionWizardView):
         if self.get_method() in ('call', 'sms'):
             self.get_device(user=self.request.user, name='default').save()
 
-        return redirect('two_factor:setup_complete')
+        return redirect(self.redirect_url)
 
     def get_form_kwargs(self, step=None):
         kwargs = {}
@@ -258,11 +267,12 @@ class SetupView(IdempotentSessionWizardView):
     def get_context_data(self, form, **kwargs):
         context = super(SetupView, self).get_context_data(form, **kwargs)
         if self.steps.current == 'generator':
-            alias = '%s@%s' % (self.request.user.username,
-                               get_current_site(self.request).name)
-            key = unhexlify(self.get_key('generator').encode('ascii'))
+            key = self.get_key('generator')
+            rawkey = unhexlify(key.encode('ascii'))
+            b32key = b32encode(rawkey).decode('utf-8')
+            self.request.session[self.session_key_name] = b32key
             context.update({
-                'QR_URL': get_qr_url(alias, key)
+                'QR_URL': reverse(self.qrcode_url)
             })
         elif self.steps.current == 'validation':
             context['device'] = self.get_device()
@@ -292,6 +302,7 @@ class BackupTokensView(FormView):
     a pillow ;-).
     """
     form_class = Form
+    redirect_url = 'two_factor:backup_tokens'
     template_name = 'two_factor/core/backup_tokens.html'
     number_of_tokens = 10
 
@@ -312,7 +323,7 @@ class BackupTokensView(FormView):
         for n in range(self.number_of_tokens):
             device.token_set.create(token=StaticToken.random_token())
 
-        return redirect('two_factor:backup_tokens')
+        return redirect(self.redirect_url)
 
 
 @class_view_decorator(never_cache)
@@ -327,6 +338,7 @@ class PhoneSetupView(IdempotentSessionWizardView):
     numbers can be used for verification.
     """
     template_name = 'two_factor/core/phone_register.html'
+    redirect_url = None
     form_list = (
         ('setup', PhoneNumberMethodForm),
         ('validation', DeviceValidationForm),
@@ -338,7 +350,7 @@ class PhoneSetupView(IdempotentSessionWizardView):
         Store the device and redirect to profile page.
         """
         self.get_device(user=self.request.user, name='backup').save()
-        return redirect(str(settings.LOGIN_REDIRECT_URL))
+        return redirect(self.redirect_url or str(settings.LOGIN_REDIRECT_URL))
 
     def render_next_step(self, form, **kwargs):
         """
@@ -399,3 +411,41 @@ class SetupCompleteView(TemplateView):
     View congratulation the user when OTP setup has completed.
     """
     template_name = 'two_factor/core/setup_complete.html'
+
+
+@class_view_decorator(never_cache)
+@class_view_decorator(login_required)
+class QRGeneratorView(View):
+    """
+    View returns an SVG image with the OTP token information
+    """
+    http_method_names = ['get']
+    default_qr_factory = 'qrcode.image.svg.SvgPathImage'
+    session_key_name = 'django_two_factor-qr_secret_key'
+
+    # The qrcode library only supports PNG and SVG for now
+    image_content_types = {
+        'PNG': 'image/png',
+        'SVG': 'image/svg+xml; charset=utf-8',
+    }
+
+    def get(self, request, *args, **kwargs):
+        # Get the data from the session
+        try:
+            key = self.request.session[self.session_key_name]
+            del self.request.session[self.session_key_name]
+        except KeyError:
+            raise Http404()
+
+        # Get data for qrcode
+        image_factory_string = getattr(settings, 'TWO_FACTOR_QR_FACTORY', self.default_qr_factory)
+        image_factory = import_by_path(image_factory_string)
+        content_type = self.image_content_types[image_factory.kind]
+        alias = '%s@%s' % (self.request.user.username,
+                           get_current_site(self.request).name)
+
+        # Make and return QR code
+        img = qrcode.make(get_otpauth_url(alias, key), image_factory=image_factory)
+        resp = HttpResponse(content_type=content_type)
+        img.save(resp)
+        return resp
